import styles from "../index.module.css";
import React, { useState} from 'react';
import { Outlet, Link, useLoaderData} from "react-router-dom";
import mqtt from 'precompiled-mqtt';
import {creerSensor, existSensor, updateSensor, Sensors} from '../sensor.js';

export async function loader() {
  const contacts = Sensors;
  return { contacts };
}
export default function Root() {
  const [sensors, setSensors] = useState([]);
  const [brokerUrl, setBrokerUrl] = useState('');

  const { contacts } = useLoaderData();

  const topic = 'value/#';
  const [client, setClient] = useState(null);
  //const client = mqtt.connect(brokerUrl);

  const [isConnected, setIsConnected] = useState(false);

  const handleUrlChange = (e) => {
    const url = e.target.value;
    setBrokerUrl(url);
  };

  const handleDisconnect = () => {
    if(isConnected) {
      client.end();
      setIsConnected(false);
    }
  };

  const handleConnect = () => {
    if(!isConnected) {
      const newClient = mqtt.connect(brokerUrl);
      newClient.on('connect', () => {
        console.log(`Connected to MQTT broker ${brokerUrl}`);
        newClient.subscribe(topic, (err) => {
          if (err) {
            console.error('Error subscisConnectedribing to topic', err);
          } else {
            console.log(`Subscribed to topic ${topic}`);
          }
        });
      });

      newClient.on('message', (topic, message) => {
        const sensorData = JSON.parse(message.toString());
        const name = sensorData.name;
        const data = sensorData.value;
        if(existSensor(name)){
          updateSensor(name, data);
        }else {
          creerSensor(sensorData);
        }
        setSensors((prevSensors) => {
          const existingSensor = prevSensors.find((sensor) => sensor.name === sensorData.name);
          if (existingSensor) {
            const updatedSensors = [...prevSensors];
            updatedSensors[existingSensor] = sensorData;
            return updatedSensors;
          } else {
            return [...prevSensors, sensorData];
          }
        });
      });
      setClient(newClient);
    }
    setIsConnected(true);
  };

  return (
    <>
      <div className={styles.wrapper}>
        <div className={styles.header}>
          <div className={styles.navbar}>
            <a href="">Real time sensors</a>
            <a href={window.location.pathname}>Other</a>
            <a href={window.location.pathname}>Foo</a>
          </div>
        </div>
        <div className={styles.sidebar}>
          <h1 className={styles.sensors}>Sensors</h1>
          <div className={styles.listSensors}>
            <nav>
            {sensors.length ? (
            <ul>
              {sensors.map((sensor) => (
                <li key={sensor.id}>
                  <Link to={`/${sensor.name}`}>
                    {sensor.name ? (
                      <>
                        {sensor.name}
                      </>
                    ) : (
                      <i>No Name</i>
                    )}{" "}
                  </Link>
                </li>
              ))}
            </ul>
          ) : (
            <p>
              <i>No sensors</i>
            </p>
          )}
            </nav>
          </div>
        </div>
        <div className={styles.divUrl}>
          <h4 className={styles.mqtt}>MQTT Brocker:</h4>
          <input type="text" className={styles.urlInput} onChange={handleUrlChange} value={brokerUrl} />
          {isConnected ? (
              <button onClick={handleDisconnect}>Disconnect</button>
          ) : (
              <button onClick={handleConnect}>Connect</button>
          )}
        </div>
        <div className={styles.details}>
          <Outlet />
        </div>
      </div>
    </>
  );
}
